__all__ = ["__version__", "version", "__version_tuple__", "version_tuple"]

from typing import Final

__version__: Final[str]
__version_tuple__: Final[tuple[int, ...]]
version: Final[str]
version_tuple: Final[tuple[int, ...]]
